@@ -3,7 +3,7 @@
 use crate::prelude::*;
 use crate::sql::*;
 use kitsune_p2p::agent_store::AgentInfoSigned;
-use kitsune_p2p::dht_arc::DhtArcSet;
+use kitsune_p2p::dht_arc::{ArcInterval, DhtArcSet};
 use kitsune_p2p::KitsuneAgent;
 use rusqlite::*;
 use std::sync::Arc;
@@ -31,6 +31,9 @@ pub trait AsP2pAgentStoreConExt {
         basis: u32,
         limit: u32,
     ) -> DatabaseResult<Vec<AgentInfoSigned>>;
+
+    /// Extrapolate coverage from agents within our own storage arc
+    fn p2p_extrapolated_coverage(&mut self, dht_arc_set: DhtArcSet) -> DatabaseResult<Vec<f64>>;
 }
 
 /// Extension trait to treat transaction instances
@@ -52,6 +55,9 @@ pub trait AsP2pStateTxExt {
 
     /// Query agents sorted by nearness to basis loc
     fn p2p_query_near_basis(&self, basis: u32, limit: u32) -> DatabaseResult<Vec<AgentInfoSigned>>;
+
+    /// Extrapolate coverage from agents within our own storage arc
+    fn p2p_extrapolated_coverage(&self, dht_arc_set: DhtArcSet) -> DatabaseResult<Vec<f64>>;
 }
 
 impl AsP2pAgentStoreConExt for crate::db::PConnGuard {
@@ -79,6 +85,10 @@ impl AsP2pAgentStoreConExt for crate::db::PConnGuard {
     ) -> DatabaseResult<Vec<AgentInfoSigned>> {
         self.with_reader(move |reader| reader.p2p_query_near_basis(basis, limit))
     }
+
+    fn p2p_extrapolated_coverage(&mut self, dht_arc_set: DhtArcSet) -> DatabaseResult<Vec<f64>> {
+        self.with_reader(move |reader| reader.p2p_extrapolated_coverage(dht_arc_set))
+    }
 }
 
 /// Put an AgentInfoSigned record into the p2p_store
@@ -242,6 +252,47 @@ impl AsP2pStateTxExt for Transaction<'_> {
         }
         Ok(out)
     }
+
+    fn p2p_extrapolated_coverage(&self, dht_arc_set: DhtArcSet) -> DatabaseResult<Vec<f64>> {
+        let mut stmt = self
+            .prepare(sql_p2p_agent_store::EXTRAPOLATED_COVERAGE)
+            .map_err(|e| rusqlite::Error::ToSqlConversionFailure(e.into()))?;
+
+        let mut out = Vec::new();
+
+        let now = std::time::SystemTime::now()
+            .duration_since(std::time::SystemTime::UNIX_EPOCH)
+            .unwrap()
+            .as_millis() as u64;
+
+        for interval in dht_arc_set.intervals() {
+            match interval {
+                ArcInterval::Full => {
+                    out.push(stmt.query_row(
+                        named_params! {
+                            ":now": now,
+                            ":start_loc": 0,
+                            ":end_loc": u32::MAX,
+                        },
+                        |r| r.get(0),
+                    )?);
+                }
+                ArcInterval::Bounded(start, end) => {
+                    out.push(stmt.query_row(
+                        named_params! {
+                            ":now": now,
+                            ":start_loc": (*start).0,
+                            ":end_loc": (*end).0,
+                        },
+                        |r| r.get(0),
+                    )?);
+                }
+                _ => (),
+            }
+        }
+
+        Ok(out)
+    }
 }
 
 /// Owned data dealing with a full p2p_agent_store record.
